
/*
 *  B L O C K S
 *
 *  Polyomino tiling
 *  Handles symmetrical pieces
 *
 *  Written by PM 2Ring  1998
 *
 *  Jul 11 1998
 */

#include <stdio.h>
#include <stdlib.h>

#include "blocks.h"

/* Polyomino structures */
SHAPE *shapes;

/* Count found solutions */
int solvecount;

/* Number of solutions desired */
int maxsolve;

/* Count succesfully tested pieces */
int testcount;

/* How often to report shape tests */
int maxtest;

/* Number of polyominoes */
int smax;

/* Maximum number of squares (cells) in polyominoes */
int pmax;

/* Board to fill with polyominoes */
char *board;
int bwidth, bheight, bbwidth, bbheight;

/* Shape input grid */
SHGRID grid;

/* Line style */
int lineflag;

/* Use ANSI colour? */
int colourflag;

/* Dump shape data before solving? */
int verbose;

/* Chars used as shape names */
char *charset;
int charsetlen;

/* Attempt to fill board. shapemap is a bitmap of used shapes. */
void fillboard(int shapemap, int depth)
{
    int q, s, newmap, found = 0;
    int spotX, spotY;
    SHAPE *sh = shapes;

    /* Search for next available spot */
    for (spotY=0; spotY<bheight && !found; spotY++)
        for (spotX=0; spotX<bwidth && !found; spotX++)
            found = *pbrd(spotX, spotY) == EMPTY;
    spotX--; spotY--;

    /* Try each shape */
    for(s=0; s<smax; s++, sh++)
    {
        newmap = 1<<s;
        if (shapemap & newmap)
            continue;
        newmap |= shapemap;

        /* Try each unique orientation */
        for (q=0; q<sh->q; q++)
        {
            if (testshape(spotX, spotY, sh, q) == OK)
            {
                /* Add shape to board */
                doshape(spotX, spotY, sh, q, ADD);

                /* Progress indicator */
                if ((++testcount % maxtest) == 0)
                    printe("\r %d ", testcount);

                if (depth == smax)
                {
                    solvecount += 1;
                    printe("\r %d tests passed. ", testcount);                    
                    printe("Solution %d.\n", solvecount);
                    
                    if (lineflag != POV)
                      printf("Solution %d:\n", solvecount);
                    putboard();
                    
                    /* Bail out if we have enough solutions */
                    if (maxsolve && solvecount == maxsolve)
                      exit(0);

                    /* Remove shape from board */
                    doshape(spotX, spotY, sh, q, REM);
                    testcount = 0;
                    return;   /* Don't check other rotations */
                }
                else  /* Redundant else */
                {
                    fillboard(newmap, depth+1);
                    doshape(spotX, spotY, sh, q, REM);
                }
            }
        }
    }
}

/* Called on exit via atexit() */
void myexit(void)
{
    freeshapes(shapes);

    free(board);
    free(grid);

    resetpen();
    
    if (lineflag == POV) 
      printf("/* End of file */\n\n");
    printe("\nFinished\n\n");
}

int main(int argc, char **argv)
{
    FILE *fp;
    char *cfgname;
 
    if (argc<1 || *argv[argc-1] == '?')
    {
        printe("Polyomino puzzle solver"
        " by PM 2Ring.\n\n"
        "Usage: %s [configfile]\n\n"
        "Default config=%s.\n\n"
        "See the config file for details\n",
            argv[0], CFGNAME);
        
        return 0;
    }

    cfgname = argc>1 ? argv[1] : CFGNAME;

    atexit(myexit);
    
    if (!(fp = fopen(cfgname, "r")))
    {
        printe("Can't open config file <%s>\n", cfgname);
        exit(10);
    }

    if (readcfg(fp) != OK)
    {
        fclose(fp);
        printe("Error reading config file <%s>\n", cfgname);
        exit(10);
    }
    fclose(fp);

    if (lineflag == POV)
      printf("/* POV include file. Polyominos \n"
      " *  Generated using %s\n */\n\n", cfgname);
     
    printe("\nSearching...\n");

    /* Do it! */
    fillboard(0, 1);

    return 0;
}

