/* General definitions and prototypes */
#include <stdio.h>              /* Needed for FILE */

/* Default config file name */
#define CFGNAME "blocks.cfg"

/* Chars used to build shapes & board */
#define FULL '*'
#define EMPTY '.'

/* Various constants */
#define OK 0                    /* Success Return code */
#define QMAX 4                  /* Rotations */

#define ES "\x1b"               /* For ANSI escape codes */

/* For Add or Remove shape flag */
enum {ADD, REM};

/* Line types */
enum
{
    NOLINE, ISO, IBM1, IBM2, POV, MAXL
};

/* Char types */
enum
{
    NOCHAR, ALPHA, NUMERIC, MAXC
};

/* Polyomino as a char grid */
typedef char *SHGRID;

/* Square coordinates */
typedef int POINT[2];

/* Polyomino structure */
typedef struct shape
{
    int id;                     /* Shape ID number */
    int p;                      /* Number of squares in this shape. */
    int w, h;                   /* Shape bounding box size */
    int q;                      /* Symmetry. Local maxq */
    POINT *cell;                /* Shape elements */
    POINT cnr[QMAX];            /* "Corner" cell for each rotation */
} SHAPE;

extern char *board;
extern int smax, pmax, bwidth, bheight, bbwidth, bbheight;
extern SHGRID grid;
extern SHAPE *shapes;

extern int maxtest;
extern int lineflag;
extern char *charset;
extern int charsetlen;
extern int colourflag;
extern int verbose;
extern int solvecount;
extern int maxsolve;

extern void (*putboard)(void);

/* Convert grid coords from 2D to 1D */
#define pgrid(g,x,y) ((g) + ((x) + (y)*pmax))

/* Convert board coords from 2D to 1D */
#define pbrd(x,y) (board + ((x)+pmax-1 + ((y)+pmax-1)*bbwidth))

/* Prototypes */
/* grid.c */
int grid2shape(SHGRID g, SHAPE *sh);
int scangrid(SHGRID g, SHAPE *sh);
void findcorners(SHGRID g, SHAPE *sh);
void fixcorners(SHGRID g, SHAPE *sh);
void rotate(POINT pt, SHAPE *sh, int q, int *x, int *y);

/* shape.c */
void doshape(int spotX, int spotY, SHAPE *sh, int q, int setf);
int testshape(int spotX, int spotY, SHAPE *sh, int q);
void dumpshape(SHAPE *sh);
void showshapes(SHAPE *shapes);

/* board.c */
void setpen(int newpen);
void resetpen(void);
void putboardL(void);
void putboardNL(void);
void putboardQ(void);
void putboardPOV(void);

/* blocks.c */
void fillboard(int shapemap, int depth);
void myexit(void);

/* config.c */
int readcfg(FILE *fp);
char *gettext(FILE *fp);
char *getline(FILE *fp);
int readnums(FILE *fp);
int readshapes(FILE *fp);
void freeshapes(SHAPE *shapes);
int printe(char *fmt, ...);
