/* Grid to shape conversion */
#include <stdlib.h>

#include "blocks.h"

/* Convert grid string array to SHAPE structure.
   Return OK on success  */
int grid2shape(SHGRID g, SHAPE *sh)
{
    /* Allocate cells */
    if (!(sh->cell = calloc(pmax, sizeof(POINT))))
        return !OK;

    if (scangrid(g, sh) != OK)
        return !OK;
    
    findcorners(g, sh);
    fixcorners(g, sh);

    return OK;
}

/* Do initial conversion from grid to shape */
int scangrid(SHGRID g, SHAPE *sh)
{
    int i, j, k = 0, w = 0, h = 0;
    static int id=0;

    /* Search for squares, marked by stars */
    for (i = 0; i < pmax && k < pmax; i++)
    {
        for (j = 0; j < pmax && k < pmax; j++)
        {
            switch (*pgrid(g,j,i))
            {
            case FULL:
                /* Store coordinates */
                sh->cell[k][0] = j;
                sh->cell[k][1] = i;
                k++;

                /* Update bounds */
                if (j > w)
                    w = j;
                if (i > h)
                    h = i;
                break;
            case ' ':
            case EMPTY:
                /* Blank */
                break;
            default:
                return !OK;       /* Illegal char in grid */
                break;
            }
        }
    }

    if (k == 0)
        return !OK;       /* No chars in grid! */

    sh->id = id++;
    sh->p = k;
    sh->w = w;
    sh->h = h;

    return OK;
}

/* Find corner cells.  These are the leading squares used to place
   the piece in each rotation */
void findcorners(SHGRID g, SHAPE *sh)
{
    int i, j, k;

    /* Copy first & last cells to top & bottom corners */
    sh->cnr[0][0] = sh->cell[0][0];
    sh->cnr[0][1] = sh->cell[0][1];

    sh->cnr[2][0] = sh->cell[sh->p-1][0];
    sh->cnr[2][1] = sh->cell[sh->p-1][1];

    /* Rescan grid for left & right corners */
    for (k = 0, j = sh->w; j >= 0 && k < sh->p; j--)
    {
        for (i = 0; i <= sh->h && k < sh->p; i++)
        {
            if (*pgrid(g,j,i) == FULL)
            {
                int kk = k==sh->p-1;

                if (k==0 || kk)
                {
                    /* Store coordinates */
                    kk = kk ? 3 : 1;
                    sh->cnr[kk][0] = j;
                    sh->cnr[kk][1] = i;
                }
                k++;
            }
        }
    }
}

/* Find symmetry of piece & pre-rotate corners */
void fixcorners(SHGRID g, SHAPE *sh)
{
    int k, q, symf;

    /* Check symmetries  */
    for (q=1; q<=2; q++)
    {
        symf = 1;
        for (k=0; symf && k<sh->p; k++)
        {
            int x, y;

            rotate(sh->cell[k], sh, q, &x, &y);
            symf = *pgrid(g,x,y) == FULL;
        }
        if (symf && k==sh->p)
            break;
    }
    sh->q = symf ? q : QMAX;

    /* Rotate corners */
    for (k=1; k<sh->q; k++)
        rotate(sh->cnr[k], sh, k, sh->cnr[k], sh->cnr[k]+1);
}

/* Rotate pt of sh by q. Store values in *x & *y */
void rotate(POINT pt, SHAPE *sh, int q, int *x, int *y)
{
    int tx, ty;

    switch(q)
    {
        case 0:
            tx = pt[0];
            ty = pt[1];
        break;
        case 1:
            tx = pt[1];
            ty = sh->w - pt[0];
        break;
        case 2:
            tx = sh->w - pt[0];
            ty = sh->h - pt[1];
        break;
        case 3:
            tx = sh->h - pt[1];
            ty = pt[0];
        break;
    }
    *x = tx;
    *y = ty;
}

